@@ -2,9 +2,9 @@
 
 """This test unit checks node attributes that are persistent (AttributeSet)."""
 
-from __future__ import print_function
 import sys
 import datetime
+import warnings
 from distutils.version import LooseVersion
 
 import numpy
@@ -16,7 +16,7 @@ from tables import (IsDescription, Int32Atom, StringCol, IntCol, Int16Col,
 from tables.exceptions import DataTypeWarning
 from tables.parameters import NODE_CACHE_SLOTS
 from tables.tests import common
-from tables.tests.common import unittest
+from tables.tests.common import unittest, test_filename
 from tables.tests.common import PyTablesTestCase as TestCase
 
 
@@ -69,6 +69,93 @@ class CreateTestCase(common.TempFileMixin, TestCase):
         self.assertEqual(self.h5file.get_node_attr(self.root.anarray, 'attr1'),
                          "n" * attrlength)
 
+    def reopen(self):
+        # Reopen
+        if self.close:
+            if common.verbose:
+                print("(closing file version)")
+            self._reopen(mode='r+', node_cache_slots=self.node_cache_slots)
+            self.root = self.h5file.root
+
+    def check_missing(self,name):
+        self.reopen()
+        self.assertNotIn(name, self.root.agroup._v_attrs)
+        self.assertNotIn(name, self.root.atable.attrs)
+        self.assertNotIn(name, self.root.anarray.attrs)
+
+
+    def check_name(self, name, val = ''):
+        """Check validity of attribute name filtering"""
+        self.check_missing(name)
+        # Using File methods
+        self.h5file.set_node_attr(self.root.agroup, name, val)
+        self.h5file.set_node_attr(self.root.atable, name, val)
+        self.h5file.set_node_attr(self.root.anarray, name, val)
+         # Check File methods
+        self.reopen()
+        self.assertEqual(self.h5file.get_node_attr(self.root.agroup, name),val)
+        self.assertEqual(self.h5file.get_node_attr(self.root.atable, name),val)
+        self.assertEqual(self.h5file.get_node_attr(self.root.anarray, name),val)
+        # Remove, file methods
+        self.h5file.del_node_attr(self.root.agroup, name)
+        self.h5file.del_node_attr(self.root.atable, name)
+        self.h5file.del_node_attr(self.root.anarray, name)
+        self.check_missing(name)
+
+        # Using Node methods
+        self.root.agroup._f_setattr(name, val)
+        self.root.atable.set_attr(name, val)
+        self.root.anarray.set_attr(name, val)
+        # Check Node methods
+        self.reopen()
+        self.assertEqual(self.root.agroup._f_getattr(name), val)
+        self.assertEqual(self.root.atable.get_attr(name), val)
+        self.assertEqual(self.root.anarray.get_attr(name), val)
+        self.root.agroup._f_delattr(name)
+        self.root.atable.del_attr(name)
+        self.root.anarray.del_attr(name)
+        self.check_missing(name)
+
+        # Using AttributeSet methods
+        setattr(self.root.agroup._v_attrs, name, val)
+        setattr(self.root.atable.attrs, name, val)
+        setattr(self.root.anarray.attrs, name, val)
+        # Check AttributeSet methods
+        self.reopen()
+        self.assertEqual(getattr(self.root.agroup._v_attrs, name), val)
+        self.assertEqual(getattr(self.root.atable.attrs, name), val)
+        self.assertEqual(getattr(self.root.anarray.attrs, name), val)
+        delattr(self.root.agroup._v_attrs,name)
+        delattr(self.root.atable.attrs, name)
+        delattr(self.root.anarray.attrs, name)
+        self.check_missing(name)
+
+        # Using dict []
+        self.root.agroup._v_attrs[name]=val
+        self.root.atable.attrs[name]=val
+        self.root.anarray.attrs[name]=val
+        # Check dict []
+        self.reopen()
+        self.assertEqual(self.root.agroup._v_attrs[name], val)
+        self.assertEqual(self.root.atable.attrs[name], val)
+        self.assertEqual(self.root.anarray.attrs[name], val)
+        del self.root.agroup._v_attrs[name]
+        del self.root.atable.attrs[name]
+        del self.root.anarray.attrs[name]
+        self.check_missing(name)
+
+    def test01a_setAttributes(self):
+        """Checking attribute names validity"""
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore', tables.NaturalNameWarning)
+            self.check_name('a')
+            self.check_name('a:b')
+            self.check_name('/a/b')
+            self.check_name('.')
+        self.assertRaises(ValueError, self.check_name, '')
+        self.assertRaises(ValueError, self.check_name, '__members__')
+        self.assertRaises(TypeError, self.check_name, 0)
+
     def test02_setAttributes(self):
         """Checking setting large string attributes (Node methods)"""
 
@@ -482,7 +569,7 @@ class CreateTestCase(common.TempFileMixin, TestCase):
         attrs = self.group._v_attrs
         attrs.pq = "1"
         del attrs['pq']
-        self.assertTrue('pq' not in attrs._f_list())
+        self.assertNotIn('pq', attrs._f_list())
 
     def test11d_KeyError(self):
         """Checking that KeyError is raised in __getitem__/__delitem__."""
@@ -515,6 +602,52 @@ class CreateTestCase(common.TempFileMixin, TestCase):
         assert_array_equal(self.array.attrs['b'], data.T)
         assert_array_equal(self.array.attrs['c'], data.T)  # AssertionError!
 
+    @unittest.skipIf(sys.version_info[0] < 3, "Special method `__dir__()` introduced in Python-3.")
+    def test12_dir(self):
+        """Checking AttributeSet.__dir__"""
+
+        if common.verbose:
+            print('\n', '-=' * 30)
+            print("Running %s.test12_dir..." % self.__class__.__name__)
+
+        attrset = self.group._v_attrs
+
+        user_attr = 'good_attr'
+        sys_attr = 'BETTER_ATTR'
+        for a in [user_attr, sys_attr]:
+            attrset[a] = 1
+
+        bad_user = '5bad'
+        bad_sys = 'SYS%'
+        for a in [bad_user, bad_sys]:
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore', tables.NaturalNameWarning)
+                attrset[a] = 1
+
+        completions = dir(attrset)
+
+        ## Check some regular attributes.
+        #
+        self.assertIn('__class__', completions)
+        self.assertIn('_f_copy', completions)
+        self.assertEqual(completions.count('_f_copy'), 1)
+
+        ## Check SYS attrs.
+        #
+        self.assertNotIn(bad_sys, completions)
+        self.assertIn(sys_attr, completions)
+        self.assertEqual(completions.count(sys_attr), 1)
+
+        ## Check USER attrs.
+        #
+        self.assertIn(user_attr, completions)
+        self.assertNotIn(bad_user, completions)
+        self.assertEqual(completions.count(user_attr), 1)
+
+        # Now check all for no duplicates.
+        self.assertSequenceEqual(sorted(set(completions)),
+                                 sorted(completions))
+
 
 class NotCloseCreate(CreateTestCase):
     close = False
@@ -555,6 +688,7 @@ class DictCacheCloseCreate(CreateTestCase):
 class TypesTestCase(common.TempFileMixin, TestCase):
 
     def setUp(self):
+        self.open_kwargs = {'allow_padding': self.allow_padding}
         super(TypesTestCase, self).setUp()
         self.root = self.h5file.root
 
@@ -609,9 +743,9 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.root.anarray.attrs.pq, numpy.bool_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.qr, numpy.bool_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.rs, numpy.bool_))
+        self.assertIsInstance(self.root.anarray.attrs.pq, numpy.bool_)
+        self.assertIsInstance(self.root.anarray.attrs.qr, numpy.bool_)
+        self.assertIsInstance(self.root.anarray.attrs.rs, numpy.bool_)
         self.assertEqual(self.root.anarray.attrs.pq, True)
         self.assertEqual(self.root.anarray.attrs.qr, False)
         self.assertEqual(self.root.anarray.attrs.rs, True)
@@ -685,9 +819,9 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.root.anarray.attrs.pq, numpy.int_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.qr, numpy.int_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.rs, numpy.int_))
+        self.assertIsInstance(self.root.anarray.attrs.pq, numpy.int_)
+        self.assertIsInstance(self.root.anarray.attrs.qr, numpy.int_)
+        self.assertIsInstance(self.root.anarray.attrs.rs, numpy.int_)
         self.assertEqual(self.root.anarray.attrs.pq, 1)
         self.assertEqual(self.root.anarray.attrs.qr, 2)
         self.assertEqual(self.root.anarray.attrs.rs, 3)
@@ -696,8 +830,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (scalar, NumPy case)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype, numpy.array(1, dtype=dtype))
@@ -723,8 +857,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (unidimensional NumPy case)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype, numpy.array([1, 2], dtype=dtype))
@@ -748,8 +882,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (unidimensional, non-contiguous)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             arr = numpy.array([1, 2, 3, 4], dtype=dtype)[::2]
@@ -774,8 +908,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Int attributes (bidimensional NumPy case)"""
 
         # 'UInt64' not supported on Win
-        checktypes = ['Int8', 'Int16', 'Int32', 'Int64',
-                      'UInt8', 'UInt16', 'UInt32']
+        checktypes = ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -817,17 +951,17 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.root.anarray.attrs.pq, numpy.float_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.qr, numpy.float_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.rs, numpy.float_))
-        self.assertTrue(self.root.anarray.attrs.pq, 1.0)
-        self.assertTrue(self.root.anarray.attrs.qr, 2.0)
-        self.assertTrue(self.root.anarray.attrs.rs, 3.0)
+        self.assertIsInstance(self.root.anarray.attrs.pq, numpy.float_)
+        self.assertIsInstance(self.root.anarray.attrs.qr, numpy.float_)
+        self.assertIsInstance(self.root.anarray.attrs.rs, numpy.float_)
+        self.assertEqual(self.root.anarray.attrs.pq, 1.0)
+        self.assertEqual(self.root.anarray.attrs.qr, 2.0)
+        self.assertEqual(self.root.anarray.attrs.rs, 3.0)
 
     def test02b_setFloatAttributes(self):
         """Checking setting Float attributes (scalar, NumPy case)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -854,7 +988,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test02c_setFloatAttributes(self):
         """Checking setting Float attributes (unidimensional NumPy case)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -881,7 +1015,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting Float attributes (unidimensional,
         non-contiguous)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             arr = numpy.array([1.1, 2.1, 3.1, 4.1], dtype=dtype)[1::2]
@@ -907,7 +1041,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test02e_setFloatAttributes(self):
         """Checking setting Int attributes (bidimensional NumPy case)"""
 
-        checktypes = ['Float32', 'Float64']
+        checktypes = ['float32', 'float64']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -976,9 +1110,9 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.root.anarray.attrs.pq, numpy.str_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.qr, numpy.str_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.rs, numpy.str_))
+        self.assertIsInstance(self.root.anarray.attrs.pq, numpy.str_)
+        self.assertIsInstance(self.root.anarray.attrs.qr, numpy.str_)
+        self.assertIsInstance(self.root.anarray.attrs.rs, numpy.str_)
         self.assertEqual(self.root.anarray.attrs.pq, 'foo')
         self.assertEqual(self.root.anarray.attrs.qr, 'bar')
         self.assertEqual(self.root.anarray.attrs.rs, 'baz')
@@ -1103,9 +1237,9 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.root.anarray.attrs.pq, numpy.complex_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.qr, numpy.complex_))
-        self.assertTrue(isinstance(self.root.anarray.attrs.rs, numpy.complex_))
+        self.assertIsInstance(self.root.anarray.attrs.pq, numpy.complex_)
+        self.assertIsInstance(self.root.anarray.attrs.qr, numpy.complex_)
+        self.assertIsInstance(self.root.anarray.attrs.rs, numpy.complex_)
         self.assertEqual(self.root.anarray.attrs.pq, 1.0 + 2j)
         self.assertEqual(self.root.anarray.attrs.qr, 2.0 + 3j)
         self.assertEqual(self.root.anarray.attrs.rs, 3.0 + 4j)
@@ -1140,7 +1274,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test05c_setComplexAttributes(self):
         """Checking setting Complex attributes (unidimensional NumPy case)"""
 
-        checktypes = ['Complex32', 'Complex64']
+        checktypes = ['complex64', 'complex128']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -1166,7 +1300,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test05d_setComplexAttributes(self):
         """Checking setting Int attributes (bidimensional NumPy case)"""
 
-        checktypes = ['Complex32', 'Complex64']
+        checktypes = ['complex64', 'complex128']
 
         for dtype in checktypes:
             setattr(self.array.attrs, dtype,
@@ -1193,9 +1327,9 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test06a_setUnicodeAttributes(self):
         """Checking setting unicode attributes (scalar case)"""
 
-        self.array.attrs.pq = 'para\u0140lel'
-        self.array.attrs.qr = ''                 # check #213 or gh-64
-        self.array.attrs.rs = 'baz'
+        self.array.attrs.pq = u'para\u0140lel'
+        self.array.attrs.qr = u''                 # check #213 or gh-64
+        self.array.attrs.rs = u'baz'
 
         # Check the results
         if common.verbose:
@@ -1214,17 +1348,17 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.array.attrs.pq, numpy.unicode_))
-        self.assertTrue(isinstance(self.array.attrs.qr, numpy.unicode_))
-        self.assertTrue(isinstance(self.array.attrs.rs, numpy.unicode_))
-        self.assertEqual(self.array.attrs.pq, 'para\u0140lel')
-        self.assertEqual(self.array.attrs.qr, '')
-        self.assertEqual(self.array.attrs.rs, 'baz')
+        self.assertIsInstance(self.array.attrs.pq, numpy.unicode_)
+        self.assertIsInstance(self.array.attrs.qr, numpy.unicode_)
+        self.assertIsInstance(self.array.attrs.rs, numpy.unicode_)
+        self.assertEqual(self.array.attrs.pq, u'para\u0140lel')
+        self.assertEqual(self.array.attrs.qr, u'')
+        self.assertEqual(self.array.attrs.rs, u'baz')
 
     def test06b_setUnicodeAttributes(self):
         """Checking setting unicode attributes (unidimensional 1-elem case)"""
 
-        self.array.attrs.pq = numpy.array(['para\u0140lel'])
+        self.array.attrs.pq = numpy.array([u'para\u0140lel'])
 
         # Check the results
         if common.verbose:
@@ -1238,7 +1372,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.array = self.h5file.root.anarray
 
         assert_array_equal(self.array.attrs.pq,
-                           numpy.array(['para\u0140lel']))
+                           numpy.array([u'para\u0140lel']))
 
     def test06c_setUnicodeAttributes(self):
         """Checking setting unicode attributes (empty unidimensional
@@ -1247,7 +1381,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         # The next raises a `TypeError` when unpickled. See:
         # http://projects.scipy.org/numpy/ticket/1037
         # self.array.attrs.pq = numpy.array([u''])
-        self.array.attrs.pq = numpy.array([''], dtype="U1")
+        self.array.attrs.pq = numpy.array([u''], dtype="U1")
 
         # Check the results
         if common.verbose:
@@ -1263,12 +1397,12 @@ class TypesTestCase(common.TempFileMixin, TestCase):
                 print("pq -->", repr(self.array.attrs.pq))
 
         assert_array_equal(self.array.attrs.pq,
-                           numpy.array([''], dtype="U1"))
+                           numpy.array([u''], dtype="U1"))
 
     def test06d_setUnicodeAttributes(self):
         """Checking setting unicode attributes (unidimensional 2-elem case)"""
 
-        self.array.attrs.pq = numpy.array(['para\u0140lel', 'bar3'])
+        self.array.attrs.pq = numpy.array([u'para\u0140lel', u'bar3'])
 
         # Check the results
         if common.verbose:
@@ -1282,7 +1416,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.array = self.h5file.root.anarray
 
         assert_array_equal(self.array.attrs.pq,
-                           numpy.array(['para\u0140lel', 'bar3']))
+                           numpy.array([u'para\u0140lel', u'bar3']))
 
     def test06e_setUnicodeAttributes(self):
         """Checking setting unicode attributes (empty unidimensional
@@ -1307,8 +1441,8 @@ class TypesTestCase(common.TempFileMixin, TestCase):
     def test06f_setUnicodeAttributes(self):
         """Checking setting unicode attributes (bidimensional 4-elem case)"""
 
-        self.array.attrs.pq = numpy.array([['para\u0140lel', 'foo2'],
-                                           ['foo3', 'para\u0140lel4']])
+        self.array.attrs.pq = numpy.array([[u'para\u0140lel', 'foo2'],
+                                           ['foo3', u'para\u0140lel4']])
 
         # Check the results
         if common.verbose:
@@ -1322,13 +1456,13 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.array = self.h5file.root.anarray
 
         assert_array_equal(self.array.attrs.pq,
-                           numpy.array([['para\u0140lel', 'foo2'],
-                                        ['foo3', 'para\u0140lel4']]))
+                           numpy.array([[u'para\u0140lel', 'foo2'],
+                                        ['foo3', u'para\u0140lel4']]))
 
     def test07a_setRecArrayAttributes(self):
         """Checking setting RecArray (NumPy) attributes."""
 
-        dt = numpy.dtype('i4,f8')
+        dt = numpy.dtype('i4,f8', align=self.aligned)
         # Set some attrs
         self.array.attrs.pq = numpy.zeros(2, dt)
         self.array.attrs.qr = numpy.ones((2, 2), dt)
@@ -1347,9 +1481,9 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.array.attrs.pq, numpy.ndarray))
-        self.assertTrue(isinstance(self.array.attrs.qr, numpy.ndarray))
-        self.assertTrue(isinstance(self.array.attrs.rs, numpy.ndarray))
+        self.assertIsInstance(self.array.attrs.pq, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.qr, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.rs, numpy.ndarray)
         assert_array_equal(self.array.attrs.pq, numpy.zeros(2, dt))
         assert_array_equal(self.array.attrs.qr, numpy.ones((2, 2), dt))
         assert_array_equal(self.array.attrs.rs, numpy.array([(1, 2.)], dt))
@@ -1377,9 +1511,9 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.array.attrs.pq, numpy.ndarray))
-        self.assertTrue(isinstance(self.array.attrs.qr, numpy.ndarray))
-        self.assertTrue(isinstance(self.array.attrs.rs, numpy.ndarray))
+        self.assertIsInstance(self.array.attrs.pq, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.qr, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.rs, numpy.ndarray)
         assert_array_equal(self.array.attrs.pq, numpy.zeros(2, dt))
         assert_array_equal(self.array.attrs.qr, numpy.ones((2, 2), dt))
         assert_array_equal(self.array.attrs.rs, numpy.array([((1, 2),)], dt))
@@ -1388,7 +1522,7 @@ class TypesTestCase(common.TempFileMixin, TestCase):
         """Checking setting multidim nested RecArray (NumPy) attributes."""
 
         # Build a nested dtype
-        dt = numpy.dtype([('f1', [('f1', 'i2', (2,)), ('f2', 'f8')])])
+        dt = numpy.dtype([('f1', [('f1', 'i2', (2,)), ('f2', 'f8')])], align=True)
 
         # Set some attrs
         self.array.attrs.pq = numpy.zeros(2, dt)
@@ -1408,21 +1542,77 @@ class TypesTestCase(common.TempFileMixin, TestCase):
             self.root = self.h5file.root
             self.array = self.h5file.root.anarray
 
-        self.assertTrue(isinstance(self.array.attrs.pq, numpy.ndarray))
-        self.assertTrue(isinstance(self.array.attrs.qr, numpy.ndarray))
-        self.assertTrue(isinstance(self.array.attrs.rs, numpy.ndarray))
+        self.assertIsInstance(self.array.attrs.pq, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.qr, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.rs, numpy.ndarray)
         assert_array_equal(self.array.attrs.pq, numpy.zeros(2, dt))
         assert_array_equal(self.array.attrs.qr, numpy.ones((2, 2), dt))
         assert_array_equal(self.array.attrs.rs, numpy.array(
             [(([1, 3], 2),)], dt))
 
+    def test08_setRecArrayNotAllowPadding(self):
+        """Checking setting aligned RecArray (NumPy) attributes with `allow_aligned` param set to False when reopen."""
+
+        dt = numpy.dtype('i4,f8', align=self.aligned)
+        # Set some attrs
+        self.array.attrs.pq = numpy.zeros(2, dt)
+        self.array.attrs.qr = numpy.ones((2, 2), dt)
+        self.array.attrs.rs = numpy.array([(1, 2.)], dt)
+
+        # Check the results
+        if common.verbose:
+            print("pq -->", self.array.attrs.pq)
+            print("qr -->", self.array.attrs.qr)
+            print("rs -->", self.array.attrs.rs)
+
+        if self.close:
+            if common.verbose:
+                print("(closing file version)")
+            self._reopen(mode='r+', allow_align=False)
+            self.root = self.h5file.root
+            self.array = self.h5file.root.anarray
+
+        self.assertIsInstance(self.array.attrs.pq, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.qr, numpy.ndarray)
+        self.assertIsInstance(self.array.attrs.rs, numpy.ndarray)
+        assert_array_equal(self.array.attrs.pq, numpy.zeros(2, dt))
+        assert_array_equal(self.array.attrs.qr, numpy.ones((2, 2), dt))
+        assert_array_equal(self.array.attrs.rs, numpy.array([(1, 2.)], dt))
+
 
 class NotCloseTypesTestCase(TypesTestCase):
-    close = 0
+    allow_padding = False
+    aligned = False
+    close = False
+
+
+class NoCloseAlignedTypesTestCase(TypesTestCase):
+    allow_padding = True
+    aligned = True
+    close = False
+
+
+class CloseNotAlignedPaddedTypesTestCase(TypesTestCase):
+    allow_padding = False
+    aligned = False
+    close = True
 
 
 class CloseTypesTestCase(TypesTestCase):
-    close = 1
+    allow_padding = True
+    aligned = False
+    close = True
+
+
+class CloseAlignedTypesTestCase(TypesTestCase):
+    allow_padding = False
+    aligned = True
+    close = True
+
+class CloseAlignedPaddedTypesTestCase(TypesTestCase):
+    allow_padding = True
+    aligned = True
+    close = True
 
 
 class NoSysAttrsTestCase(common.TempFileMixin, TestCase):
@@ -1497,7 +1687,7 @@ class NoSysAttrsClose(NoSysAttrsTestCase):
 
 
 class CompatibilityTestCase(common.TestFileMixin, TestCase):
-    h5fname = TestCase._testFilename('issue_368.h5')
+    h5fname = test_filename('issue_368.h5')
 
     @unittest.skipIf(LooseVersion(numpy.__version__) < '1.9.0',
                      'requires numpy >= 1.9')
@@ -1517,11 +1707,11 @@ class CompatibilityTestCase(common.TestFileMixin, TestCase):
         # behaviour of PyTables.
 
         self.assertEqual(
-            self.h5file.get_node_attr('/', 'py2_pickled_unicode'), 'abc')
+            self.h5file.get_node_attr('/', 'py2_pickled_unicode'), u'abc')
 
 
 class PicklePy2UnpicklePy3TestCase(common.TestFileMixin, TestCase):
-    h5fname = TestCase._testFilename('issue_560.h5')
+    h5fname = test_filename('issue_560.h5')
 
     @unittest.skipIf(sys.version_info[0] == 3 and sys.version_info[1] < 4,
                      'bug not fixed on python3<=3.3.')
@@ -1533,12 +1723,18 @@ class PicklePy2UnpicklePy3TestCase(common.TestFileMixin, TestCase):
         # a UnicodeDecodeError when unpickling on python 3.
         # Python 3.4 adds encoding='bytes' to fix this
         # http://bugs.python.org/issue6784
-        # This is not limited to datetime.datetime()
+        # Objects pickled in the testfile have non-ascii chars in the
+        # picklestring and will throw UnicodeDecodeError when unpickled
+        # on python 3.
 
+        # datetime will be unpickled with encoding='bytes'
         self.assertIsInstance(
             self.h5file.get_node_attr('/', 'py2_pickled_datetime'),
             datetime.datetime)
-
+        # dict will be unpickled with encoding='latin1'
+        d = self.h5file.get_node_attr('/', 'py2_pickled_dict')
+        self.assertIsInstance(d, dict)
+        self.assertEqual(d['s'], u'just a string')
 
 class SegFaultPythonTestCase(common.TempFileMixin, TestCase):
 
@@ -1561,7 +1757,7 @@ class EmbeddedNullsTestCase(common.TempFileMixin, TestCase):
     # See laso gh-371 (https://github.com/PyTables/PyTables/issues/371)
 
     def test_unicode(self):
-        value = "string with a null byte \x00 in it"
+        value = u"string with a null byte \x00 in it"
 
         self.h5file.root._v_attrs.name = value
         self.assertEqual(self.h5file.root._v_attrs.name, value)
@@ -1584,7 +1780,7 @@ class EmbeddedNullsTestCase(common.TempFileMixin, TestCase):
 class VlenStrAttrTestCase(TestCase):
     def setUp(self):
         super(VlenStrAttrTestCase, self).setUp()
-        self.h5fname = self._testFilename('vlstr_attr.h5')
+        self.h5fname = test_filename('vlstr_attr.h5')
         self.h5file = tables.open_file(self.h5fname)
 
     def tearDown(self):
@@ -1621,7 +1817,7 @@ class VlenStrAttrTestCase(TestCase):
 
 
 class UnsupportedAttrTypeTestCase(common.TestFileMixin, TestCase):
-    h5fname = TestCase._testFilename('attr-u16.h5')
+    h5fname = test_filename('attr-u16.h5')
 
     def test00_unsupportedType(self):
         """Checking file with unsupported type."""
@@ -1650,21 +1846,6 @@ class SpecificAttrsTestCase(common.TempFileMixin, TestCase):
             print("EXTDIM-->", ea.attrs.EXTDIM)
         self.assertEqual(ea.attrs.EXTDIM, 0)
 
-    @unittest.skipIf(sys.version_info >= (3,), 'requires Python 2')
-    def test02_unicode_title(self):
-        # see gh-370 and gh-374
-        title = 'Elevation \N{MINUS SIGN}200m'
-        self.h5file.root._v_title = title
-        self.assertEqual(self.h5file.root._v_title, title)
-
-    @unittest.skipIf(sys.version_info >= (3,), 'requires Python 2')
-    def test03_unicode_title_reopen(self):
-        # see gh-370 and gh-374
-        title = 'Elevation \N{MINUS SIGN}200m'
-        self.h5file.root._v_title = title
-        self._reopen()
-        self.assertEqual(self.h5file.root._v_title, title)
-
 
 def suite():
     theSuite = unittest.TestSuite()
@@ -1679,6 +1860,10 @@ def suite():
         theSuite.addTest(unittest.makeSuite(DictCacheCloseCreate))
         theSuite.addTest(unittest.makeSuite(NotCloseTypesTestCase))
         theSuite.addTest(unittest.makeSuite(CloseTypesTestCase))
+        theSuite.addTest(unittest.makeSuite(CloseNotAlignedPaddedTypesTestCase))
+        theSuite.addTest(unittest.makeSuite(NoCloseAlignedTypesTestCase))
+        theSuite.addTest(unittest.makeSuite(CloseAlignedTypesTestCase))
+        theSuite.addTest(unittest.makeSuite(CloseAlignedPaddedTypesTestCase))
         theSuite.addTest(unittest.makeSuite(NoSysAttrsNotClose))
         theSuite.addTest(unittest.makeSuite(NoSysAttrsClose))
         theSuite.addTest(unittest.makeSuite(CompatibilityTestCase))
